from aiida import load_profile
from aiida.plugins import WorkflowFactory
from aiida.engine import submit
from aiida.orm import *
from aiida_quantumespresso.common.types import ElectronicType, RelaxType

load_profile()

PwBase     = WorkflowFactory('quantumespresso.pw.base')
PwRelax    = WorkflowFactory('quantumespresso.pw.relax')
HpWork     = WorkflowFactory('quantumespresso.hp.main')
ScfHubbard = WorkflowFactory('quantumespresso.hp.hubbard')


def scf_hubbard(
    struct_id, 
    skip_relax=True,  
    gamma_only=True,
    qdist = None,
    kdist = None,
    hubbard_list=None,
    hubbard_file_pk=None,
    clean=True,
):
    """Function which submits a scf hubbard U+V workchain."""
    pw_codename = 'pw_7.0@hlrn-gottingen'
    hp_codename = 'hp_7.0@hlrn-gottingen'
    
    pw_code = Code.get_from_string(pw_codename)
    hp_code = Code.get_from_string(hp_codename)
    
    structure = load_node(struct_id)

    pseudo = 'SSSP/1.1/PBEsol/precision'

    options = {
        'electronic_type':ElectronicType.INSULATOR,
        'relax_type':RelaxType.POSITIONS,
    }
    
    options_dict = { 
        'meta_parameters': {
            'conv_thr_per_atom': 1e-12,
            'etot_conv_thr_per_atom': 1e-06,
        },
        'kpoints_distance':0.2,
        'pw': {
            'parameters': {
                'CONTROL': {'forc_conv_thr': 5.0e-5, 'nstep':500},
                'SYSTEM': {
                    'ecutwfc':90, 
                    'ecutrho':90*12,
                    'lda_plus_u':True,
                    'lda_plus_u_kind':2,
                    'u_projection_type': 'ortho-atomic',
                },
                'ELECTRONS': {
                    'electron_maxstep':200, 
                    'mixing_ndim':20,
                    'mixing_beta':0.7, 
                },
                'IONS': {
                    'bfgs_ndim':4,
                }, 
            },
            'metadata':{
                'options':{
                    'max_wallclock_seconds':12*60*60,
                    'resources': {'num_machines': 1},
                    'account':'hbc00053',
                },
            },
            'settings': {'cmdline':['-nk', '8']},
        },
        'pseudo_family':pseudo,
    }    

    overrides = {'clean_workdir':True, 'base':options_dict}

    relax_builder = PwRelax.get_builder_from_protocol(
        code=pw_code,
        structure=structure, 
        overrides=overrides, 
        **options
    )

    relax_builder.pop('base_final_scf', None)
    relax_builder.pop('structure')
    relax_builder.meta_convergence = Bool(True)-------------------------------
    
    options_scf = {
        'electronic_type':ElectronicType.INSULATOR
    }
    
    options_dict_scf = { 
        'meta_parameters': {
            'conv_thr_per_atom': 1e-15,
        },
        'kpoints_distance':kdist,
        'pw': {
            'parameters': {
                'SYSTEM': {
                    'ecutwfc':65, 
                    'ecutrho':12*65,
                    'lda_plus_u':True,
                    'lda_plus_u_kind':2,
                    'u_projection_type': 'ortho-atomic',
                },
            },
            'metadata':{
                'options':{
                    'max_wallclock_seconds':12*60*60,
                    'resources': {'num_machines': 1},
                    'account':'hbc00053',
                },
            },
            'settings': {'cmdline':['-nk', '4']},
        },
        'pseudo_family':pseudo,
    }    

    overrides_scf = {'clean_workdir':False, **options_dict_scf}
    scf_builder = PwBase.get_builder_from_protocol(
        code=pw_code, structure=structure, 
        overrides=overrides_scf, **options_scf
    )
     
    scf_builder.pw.pop('structure')

    hp_parameters = Dict(dict={
        "INPUTHP": {
            'conv_thr_chi': 1.e-5,
            'nmix': 4,
            'alpha_mix':0.4,
            'niter_max':150,
        }
    })
    qpoints = KpointsData()
    if gamma_only==False:
        qpoints.set_cell_from_structure(structure)
        qpoints.set_kpoints_mesh_from_density(qdist)
    else:
        qpoints.set_kpoints_mesh([1, 1, 1])

    hp_builder               = HpWork.get_builder()
    hp_builder.hp.code       = hp_code  
    hp_builder.hp.parameters = hp_parameters
    hp_builder.hp.qpoints    = qpoints
    hp_builder.parallelize_atoms = Bool(False)  
    hp_builder['hp']['metadata']['options'] = {
        'max_wallclock_seconds':12*60*60,
        'resources': {'num_machines':2},
        'account':'hbc00053',
    }
    hp_builder['hp']['settings'] = Dict(dict={'cmdline':['-nk', '8']})
    
    builder = ScfHubbard.get_builder()

    builder.structure = structure
    
    if not hubbard_list==None:
        builder.hubbard_start = List(list=hubbard_list)
    elif not hubbard_file_pk==None:
        builder.hubbard_file = load_node(hubbard_file_pk)
    else:
        return('Missing hubbard set up.')
    
    builder.scf       = scf_builder
    builder.relax     = relax_builder
    builder.hubbard   = hp_builder
    builder.meta_convergence = Bool(True)
    builder.tolerance_u      = Float(0.01)
    builder.tolerance_v      = Float(0.005)
    builder.max_iterations   = Int(7)
    builder.skip_first_relax = Bool(skip_relax)
    builder.clean_workdir = Bool(clean)

    calc = submit(builder)
    
    return calc.pk

def main():
    """
    Main function to be called to start the script
    """
    structure_id = 'c7485666-d5e7-45a4-a1d6-51343ca0d61e'
    hubbard_file_pk = '0f09f35b-9f3d-4fed-96d2-67940a6b6891'
    skip_relax = True
    clean = True
    kdist, qdist = 0.3, 0.4

    calcs = scf_hubbard(
        struct_id=structure_id, 
        hubbard_file_pk=hubbard_file_pk,
        skip_relax=skip_relax,
        gamma_only=False,
        kdist=kdist,
        qdist=qdist,
        clean=clean,
    )
    
    print(calcs)


if __name__ == '__main__':
    main()
    
