from aiida import orm, load_profile
from aiida.plugins import WorkflowFactory
from aiida_quantumespresso.common.types import ElectronicType, RelaxType
from aiida.engine import submit

load_profile()
PwRelaxWorkChain = WorkflowFactory('quantumespresso.pw.relax')


def relax(functional):
    """Relax LiNbO3 using a hybrid functional."""
    code = orm.Code.get_from_string('pw_7.0@hlrn-gottingen')
    structure = orm.load_node('c7485666-d5e7-45a4-a1d6-51343ca0d61e')
    
    pseudo = "PseudoDojo/0.4/PBE/SR/standard/upf"
    
    pw_overrides = {
        'pseudo_family': pseudo,
        'kpoints_distance':0.3,
        'meta_parameters': {
            'conv_thr_per_atom':1.0e-11,
        },
        'pw':{
            'parameters':{
                'CONTROL': {
                    'forc_conv_thr': 5.0e-5,
                    'nstep':100,
                },
                'SYSTEM': {
                    'ecutwfc':90, 
                    'ecutrho':90*4,
                    'ecutfock':90*1,
                    'input_dft': functional,
                    'nqx1':1,
                    'nqx2':1,
                    'nqx3':1,
                },
                'ELECTRONS': {
                    'adaptive_thr': True,
                    'electron_maxstep':200, 
                    'mixing_beta':0.7, 
                    'mixing_ndim':20,
                },
                'IONS': {
                    'bfgs_ndim':8,
                },
            },
            'metadata':{
                    'options':{
                        'max_wallclock_seconds':48*60*60,
                        'queue_name':'medium40',
                        'account':'hbc00053',
                        'resources': {
                            'num_machines': 1,
                            'num_mpiprocs_per_machine':40,
                            'num_cores_per_mpiproc':1,
                        },
                    },
                },
            'settings': {
                'cmdline':['-nk', '1'],
            },
        },
    }

    overrides = {'base':pw_overrides, 'clean_workdir':True,}
    
    kwargs = {
        'code':code,
        'structure':structure, 
        'electronic_type':ElectronicType.INSULATOR,
        'relax_type':RelaxType.POSITIONS,
    } 

    builder = PwRelaxWorkChain.get_builder_from_protocol(overrides=overrides, **kwargs)
    builder.pop('base_final_scf', None)

    builder.metadata.label = 'LiNbO3'
    builder.metadata.description = f'LiNbO3 relaxation with {functional}'

    calc = submit(builder)
    print(f'# {functional} : {calc.pk}')


if __name__ == '__main__':
    """Run the script"""
    functionals = ['hse', 'pbe0']
    
    for functional in functionals:
        relax(functional)