from aiida import orm, load_profile
from aiida.plugins import WorkflowFactory
from aiida_quantumespresso.common.types import ElectronicType, RelaxType
from aiida.engine import submit

load_profile("bastonero")
PwRelaxWorkChain = WorkflowFactory('quantumespresso.pw.relax')

# Init structure: 159165 (lda) - 163959 (exp)

# PBEsol : 
# .. [vc-relax] - 163436
# .. [relax] - 163760 [lda], 164113 [exp]

def relax():
    """Relax LiNbO3 using a PBEsol."""
    code = orm.Code.get_from_string('pw_7.0@hlrn-gottingen')
    structure = orm.load_node(163959)
    
    pseudo = "SSSP/1.1/PBEsol/precision"
        
    pw_overrides = {
        'pseudo_family': pseudo,
        'kpoints_distance':0.3,
        'meta_parameters': {
            'conv_thr_per_atom':1.0e-12,
        },
        'pw':{
            'parameters':{
                'CONTROL': {'forc_conv_thr': 5.0e-5, 'nstep':500},
                'SYSTEM': {
                    'ecutwfc':90, 
                    'ecutrho':90*12,
                },
                'ELECTRONS': {
                    'electron_maxstep':600, 
                    'mixing_beta':0.4, 
                    # 'mixing_ndim':20,
                },
                'IONS': {'bfgs_ndim':5}, # 'trust_radius_min':1e-4, 'trust_radius_ini':1e-4, 
                'CELL': {'press_conv_thr': 0.1, },
            },
            'settings':  {'cmdline':['-nk', '8']},
        },
        'metadata':{
            'options':{
                'resources':{
                    'num_machines':1,
                    'num_mpiprocs_per_machine':96,
                },
                'max_wallclock_seconds' : 12*60*60, 
                'account':'hbc00053',
            }
        }
    }

    overrides = {'base':pw_overrides, 'clean_workdir':True,}
    kwargs = {
        'code':code,
        'structure':structure, 
        'relax_type':RelaxType.POSITIONS,
        'electronic_type':ElectronicType.INSULATOR,
    } 

    group = orm.load_group('LiNbO3')

    # From protocol
    builder = PwRelaxWorkChain.get_builder_from_protocol(overrides=overrides, **kwargs)
    builder.pop('base_final_scf', None)

    builder.metadata.label = 'LiNbO3'
    builder.metadata.description = f'LiNbO3 relaxation with PBEsol'

    # -- SUBMIT
    calc = submit(builder)
    group.add_nodes(calc)
    print(f'# PBEsol : {calc.pk}')


if __name__ == '__main__':
    """Run the script"""
    
    relax()