from aiida import orm, load_profile
from aiida.plugins import WorkflowFactory
from aiida_quantumespresso.common.types import ElectronicType
from aiida.engine import submit

from copy import deepcopy

load_profile()

PwBaseWorkChain = WorkflowFactory('quantumespresso.pw.base')
DielectricWorkChain = WorkflowFactory('vibroscopy.dielectric')
SpectraWorkChain = WorkflowFactory('vibroscopy.spectra.iraman')


def get_overrides(functional, nmach, nmpi, nk, queue, seconds, **kwargs):
    """Get the overrides."""
    cut = kwargs['cut'] # Ry
    kdist = kwargs['kdist']

    settings = {'cmdline':['-nk', str(nk)]}
     
    param_electrons = {
        'electron_maxstep':200,
        'mixing_beta':0.7,
    }

    if functional=="LDA":
        pseudo = 'PseudoDojo/0.4/LDA/SR/stringent/upf'
        param_system =  {
            'ecutwfc':cut,
            'ecutrho':cut*4
        }
    elif functional=="PBEsol":
        pseudo = "SSSP/1.1/PBEsol/efficiency"
        param_system =  {
            'ecutwfc':cut,
            'ecutrho':cut*12
        }
    elif functional=="PBEsol+U":
        pseudo = "SSSP/1.1/PBEsol/efficiency"
        param_system =  {
            'ecutwfc':cut,
            'ecutrho':cut*12,
            'lda_plus_u':True,
            'lda_plus_u_kind':0,
            'u_projection_type': 'ortho-atomic',
            'hubbard_u':{'Nb':3.25},
        }
    elif functional=="PBEsol+U+V":
        pseudo = "SSSP/1.1/PBEsol/efficiency"
        param_system =  {
            'ecutwfc':cut,
            'ecutrho':cut*12,
            'lda_plus_u':True,
            'lda_plus_u_kind':2,
            'u_projection_type': 'ortho-atomic',
            'hubbard_parameters':'file',
        }
    elif functional=="PBE":
        pseudo = "SSSP/1.1/PBE/efficiency"
        param_system =  {
            'ecutwfc':cut,
            'ecutrho':cut*12
        }
    elif functional=="PBE0" or functional=="HSE06":
        pseudo = 'PseudoDojo/0.4/PBE/SR/standard/upf'
        param_system =  {
            'ecutwfc':cut,
            'ecutrho':cut*4,
            'ecutfock':cut*1,
            'nqx1':1,
            'nqx2':1,
            'nqx3':1,
        }
        if functional=="PBE0":
            param_system["input_dft"] = 'pbe0'
        if functional=="HSE06":
            param_system["input_dft"] = 'hse'
        param_electrons = {
            'electron_maxstep':200,
            'mixing_beta':0.4,
            'diagonalization': 'ppcg',
            'mixing_ndim':20,
            'adaptive_thr': True,
        }
        settings =  {'cmdline':['-nk', str(nk),'-ndiag','1']}
    else:
        print("ERRORE PSEUDO")
        return

    param_control = {}

    overrides = {
        'pseudo_family':pseudo,
        'clean_workdir':True,
        'kpoints_distance':kdist,
        'meta_parameters': {
            'conv_thr_per_atom':kwargs['convthr'],
        },
        'pw':{
            'parameters':{
                'CONTROL': param_control,
                'SYSTEM': param_system,
                'ELECTRONS': param_electrons,
            },
            'settings':  settings,
            'metadata':{
                'options':{
                    'resources':{
                        'num_machines':nmach,
                        'num_mpiprocs_per_machine':nmpi,
                    },
                    'max_wallclock_seconds' : seconds,
                    'queue_name': queue,
                    'account':'hbi00059',
                }
            }
        }
    }

    return overrides

def spectra(structure_pk, functional, **kwargs):
    """
    Launch a spectra finite displacements and fields calculation.
    """
    code = orm.Code.get_from_string('pw_6.8@hlrn-gottingen')
    structure = orm.load_node(structure_pk)

    kwargs2 = {
        'code':code,
        'structure':structure,
        'electronic_type':ElectronicType.INSULATOR,
    }

    # functional, nmach, nmpi, nk, queue, seconds
    fkwargs = deepcopy(kwargs)
    fkwargs['kdist'] = 0.15
    fkwargs['convthr'] = 1e-12
    forces_overrides = get_overrides(functional, 1, 40, 1, 'medium40', 24*60*60, **fkwargs)
    diel_overrides = get_overrides(functional, 1, 40, 1, 'large40', 24*60*60, **kwargs)

    scf_forces_builder = PwBaseWorkChain.get_builder_from_protocol(overrides=forces_overrides, **kwargs2)
    scf_diel_builder = PwBaseWorkChain.get_builder_from_protocol(overrides=diel_overrides, **kwargs2)

    if functional=="PBEsol+U+V":
        scf_forces_builder.pw.hubbard_file = orm.load_node('aab8b8c2-2e9e-4f80-a5e4-4ababca4633e')
        scf_diel_builder.pw.hubbard_file = orm.load_node('aab8b8c2-2e9e-4f80-a5e4-4ababca4633e')

    scf_forces_builder.pop('clean_workdir')
    scf_forces_builder['pw'].pop('structure')
    scf_diel_builder.pop('clean_workdir')

    diel_builder = DielectricWorkChain.get_builder()
    diel_builder.scf = scf_diel_builder
    diel_builder.property = 'raman'
    diel_builder.kpoints_parallel_distance = orm.Float(kwargs['kdistpar'])
    diel_builder.central_difference.electric_field_step = orm.Float(kwargs['estep'])
    diel_builder.central_difference.accuracy = orm.Int(kwargs['accuracy'])
    diel_builder.pop('symmetry')
    scf_diel_builder['pw'].pop('structure')
    diel_builder['scf']['pw'].pop('structure')

    inputs = {
        'structure': structure,
        'phonon':{
            'scf': {**scf_forces_builder},
        },
        'dielectric': {**diel_builder},
        'clean_workdir': orm.Bool(False),
    }

    calc = submit(SpectraWorkChain, **inputs)
    print(f"# {functional} @ {kwargs['cut']} Ry - {kwargs['kdist']} kd - {kwargs['kdistpar']} k// - {kwargs['convthr']} : {calc.pk}")


if __name__ == "__main__":
    """Main function to be called."""
    structure_pk = [
        '3ae6a928-3bb6-4ca4-a6a9-92395b8a3df0'
        '4fe6f279-7863-4134-bfed-90c99d25b97b'
        '6d946e28-6331-4350-b50d-c83bece018f6'
        'dfa91a32-263e-49e2-a874-157e1782dc93'
        '49831e0f-bb32-41ac-8c3d-80e1b8043c01'
        '0982a8c8-d24e-46dd-a028-7b36e9fcae98'
        '8e509508-0d22-44b3-bf0e-6e9b40e457e4'
    ]

    functionals = [
        "LDA",
        "PBE",
        "PBEsol",
        "PBEsol+U",
        "PBEsol+U+V",
        "PBE0",
        "HSE06",
    ]

    kwargs = {
        'cut': 90, # Ry
        'kdist': 0.3,
        'kdistpar': 0.15,
        'convthr': 1e-12,
        'estep': 0.0005,
        'accuracy': 2,
    }

    for pk, functional in zip(structure_pk, functionals):
        spectra(pk, functional, **kwargs)
        
